/**********************************************************************
*  Name    : Programme principal Gradateur Nunes                      *
*  Author  : Nunes Dos Anjos Carlos                                   *
*  Notice  : Copyright (c) 2005 Nunes                                 *
*          : All Rights Reserved                                      *
*  Date    : 20/02/2013                                               *
*  Version : 1.0                                                      *
*  Notes   : pour piloter les clairages du couloir et de l'entre    *
*          : Gradateur  dmarrage progressif                         *
*                                                                     *
**********************************************************************/

/** I N C L U D E S . H *********************************************/
#include "configMicro.h"
#include "p18f24k22.h"
#include "delays.h"
#include "eep.h"
#include "timebase.h"
#include "main.h"
#include "grad.h"


/** M E M O I R E   E E P R O M *************************************/
#pragma romdata eep = 0xf00000

//** Valeurs par dfaut dans eeprom *********************************/
rom unsigned char data1[] = {60, 60, 20, 20};
							// dure 0: clairage couloir d'environ 60mn + 6mn sans le cavalier 4JMP3
							// dure 1: clairage entre d'environ 60mn + 6mn sans le cavalier 4JMP1
							// dure 2: mmoire rserve pour l'clairage couloir avec le cavalier 4JMP3
							// dure 3: mmoire rserve pour l'clairage entre avec le cavalier 4JMP1


/** I N T E R R U P T I O N S ***************************************/
// High priority interrupt vector
#pragma code InterruptVectorHigh = 0x08
void InterruptVectorHigh(void)
{
	_asm
		goto InterruptServiceHigh	//jump to interrupt routine
	_endasm
}


/*********************************************************************
*** DEBUT PROGRAMME **************************************************
*********************************************************************/

#pragma code
void main (void)
{
    // initialisation porta
    PORTA = 0;
    ANSELA = 0;	// digital I/O
    TRISA = 0b11111111;	// configurations I/O des pins

    // initialisation portb
    PORTB = low;
    TRISB = 0xff;	// toutes les pins en entres
    INTCON2bits.RBPU = low;
    WPUB = 0xff;

    // initialisation portc
    ANSELC = 0;		// digital I/O
    TRISC = low;	// toutes les pins en sorties
    PORTC = 0b01111100;

    // slection frquence interne
    OSCCON = 0b01100000;	// IRCFx = 110 donc 8Mhz
    OSCTUNEbits.PLLEN = 1;  // x4 PLL

    // initialisation Timer2
    PR2 = prval;
    T2CON = init_timer2;

    // initialisation interruption
    RCONbits.IPEN = low;	// activation interruptions sans priorit
    PIR1bits.TMR2IF = low;	// effacement drapeau
    INTCONbits.PEIE = high;	// activation interruptions priphriques
    PIE1bits.TMR2IE = high;	// activation tmr2 interruption
    INTCONbits.GIE = high;	// Low priority interrupts allowed


/** Retour aprs mmorisation eeprom ********************************/
    do	// boucle 2
    {
        if (in_select) val_duree_couloir = Read_b_eep(0), val_duree_entree = Read_b_eep(1);	// rappel dure clairage couloir / entree
        else val_duree_couloir = Read_b_eep(2), val_duree_entree = Read_b_eep(3);	// rappel dure clairage couloir

        led_ok_on();
        grad_off();
        indic0bits.front3 = high;
        temps100ms_0 = 0;


/** Dbut du cycle **************************************************/
        do	// boucle 1
        {
            clrwdt();
            if (indic0bits.front3 && (temps100ms_0 > 6)) led_ok_off(), indic0bits.front3 = low;

            if (temps1mn_2 == 60) temps1h_2++, temps1s_2 = 0, temps1mn_2 = 0;   // horloge
            if (temps1h_2 == 4) indic0bits.veille = low;

            if (!in_couloir && !indic0bits.front4)
            {
                Delay10KTCYx(40);	// Delay 40 x 10000 = 400,000 cycles; 50ms @ 32MHz
                if (!in_couloir)
                {
                    temps100ms_0 = 0;
                    while ((!in_couloir) && (etape_couloir == 3))
                    {
                        if (temps100ms_0 >= val_bp_t)
                        {
                            etape_couloir = 2;
                            break;
                        }
                        clrwdt();
                    }
                    if (!etape_couloir)
                    {
                        led_couloir_on();
                        etape_couloir = 1;
                        val0_largeur_grad = val_puissancemini;
                        grad_couloir_on();
                        temps1ms_0 = 0;
                    }
                    else if (etape_couloir == 3) etape_couloir = 5;
                    else if (etape_couloir == 4)
                    {
                        indic0bits.front0 = low;
                        etape_couloir = 1;
                        temps1ms_0 = 0;
                    }
                }
            }

            if (!in_entree && !indic0bits.front5)
            {
                Delay10KTCYx(40);	// Delay 40 x 10000 = 400,000 cycles; 50ms @ 32MHz
                if (!in_entree)
                {
                    temps100ms_0 = 0;
                    while ((!in_entree) && (etape_entree == 3))
                    {
                        if (temps100ms_0 >= val_bp_t)
                        {
                            etape_entree = 2;
                            break;
                        }
                        clrwdt();
                    }
                    if (!etape_entree)
                    {
                        led_entree_on();
                        etape_entree = 1;
                        val1_largeur_grad = val_puissancemini;
                        val_puissance_entree = memo1_val_puissance;
                        grad_entree_on();
                        temps1ms_1 = 0;
                    }
                    else if (etape_entree == 3) etape_entree = 5;
                    else if (etape_entree == 4)
                    {
                        indic0bits.front1 = low;
                        etape_entree = 1;
                        temps1ms_1 = 0;
                    }
                }
            }

            if (!in_veille && !indic0bits.front2)
            {
                Delay10KTCYx(40);	// Delay 40 x 10000 = 400,000 cycles; 50ms @ 32MHz
                if (!in_veille)
                {
                    if (etape_entree == 3) {                        
                        etape_entree = 4;
                        temps1ms_1 = 0;
                        //indic0bits.veille = high;
                        //if (temps1h_2 < 4) indic0bits.veille = low;
                    }
                    else if (!etape_entree && !indic0bits.veille) {
                        led_entree_on();
                        etape_entree = 1;
                        val1_largeur_grad = val_puissancemini;
                        val_puissance_entree = val_puissancemini2;
                        grad_entree_on();
                        temps1ms_1 = 0;
                        indic0bits.front2 = high;
                        indic0bits.veille = high;
                        temps1h_2 = 0, temps1mn_2 = 0, temps1s_2 = 0;
                    }
                }
            }
            else if (in_veille && (etape_entree == 3)) indic0bits.front2 = low;

// Il faut absolument que les 2 conditions soient places ici  cause du front pour le dmarrage progessif
// sinon le programme va excuter le delay de 50ms situ juste aprs la condition de in_couloir et in_entree
// et il ne pourra plus lire le temps1ms_x  temps.
            if (in_couloir) indic0bits.front4 = low;
            if (in_entree) indic0bits.front5 = low;


/** Programme etapes ************************************************/
            switch (etape_couloir)
            {
                case 1:	// dmarrage progressif
                    indic0bits.front4 = high;
                    if ((temps1ms_0 == val_demar_progressif) && (val0_largeur_grad > memo0_val_puissance))
                    {
                        val0_largeur_grad--;
                        temps1ms_0 = 0;
                    }
                    else if (val0_largeur_grad <= memo0_val_puissance) etape_couloir = 3, temps1s_0 = 0, temps1mn_0 = 0, temps1h_0 = 0;
                    break;

                case 2:	// Changement de puissance
                    if (val0_largeur_grad == val_puissancemax) val0_largeur_grad = val_mipuissance;
                    else if (val0_largeur_grad == val_mipuissance) val0_largeur_grad = val_puissancemini1;
                    else if (val0_largeur_grad == val_puissancemini1) val0_largeur_grad = val_puissancemini2, temps1s_0 = 0, temps1mn_0 = 0, temps1h_0 = 0;
                    else if (val0_largeur_grad == val_puissancemini2) val0_largeur_grad = val_puissancemax, temps1s_0 = 0, temps1mn_0 = 0;

                    memo0_val_puissance = val0_largeur_grad;
                    indic0bits.front4 = high;
                    etape_couloir = 3;
                    break;

                case 3:	// Minuterie
                    // Etape d'attente
                    if (val0_largeur_grad == val_puissancemini2)
                    {
                        if (temps1mn_0 == 60) temps1h_0++, temps1s_0 = 0, temps1mn_0 = 0;
                        if ((temps1h_0 == 3) && (temps1mn_0 == 30))
                        {
                            etape_couloir = 4;
                            temps1ms_0 = 0;
                        }
                    }
                    else
                    {
                        if ((temps1mn_0 == val_duree_puissancemax) && (val0_largeur_grad == val_puissancemax) && !indic0bits.front0)
                            {
                                indic0bits.front0 = high;
                                val0_largeur_grad = val_mipuissance;
                            }
                            if (temps1mn_0 >= (val_duree_couloir + val_duree_puissancemax))
                            {
                                if (val0_largeur_grad == val_puissancemax) val0_largeur_grad = val_mipuissance;
                                indic0bits.front4 = high;
                                etape_couloir = 4;
                                temps1ms_0 = 0;
                            }
                    }
                    break;

                case 4:	// Arrt dgressif
                    if ((temps1ms_0 == val_arret_degressif) && (val0_largeur_grad < val_puissancemini))
                    {
                        val0_largeur_grad++;
                        temps1ms_0 = 0;
                    }
                    else if (val0_largeur_grad >= val_puissancemini) etape_couloir = 5, indic0bits.front4 = high;
                    break;

                case 5:	// Arrt
                    led_couloir_off();
                    grad_couloir_off();
                    etape_couloir = 0;
                    indic0bits.front4 = high;
                    indic0bits.front0 = low;
                    break;

                default:
                    etape_couloir = 0;
                    break;
            }

            switch (etape_entree)
            {
                case 1:	// Dmarrage progressif
                    indic0bits.front5 = high;
                    if ((temps1ms_1 == val_demar_progressif) && (val1_largeur_grad  > val_puissance_entree))
                    {
                        val1_largeur_grad--;
                        temps1ms_1 = 0;
                    }
                    else if (val1_largeur_grad <= val_puissance_entree) etape_entree = 3, temps1s_1 = 0, temps1mn_1 = 0, temps1h_1 = 0;
                    break;

                case 2:	// Changement de puissance
                    if (val1_largeur_grad == val_puissancemax) val1_largeur_grad = val_mipuissance;
                    else if (val1_largeur_grad == val_mipuissance) val1_largeur_grad = val_puissancemini1;
                    else if (val1_largeur_grad == val_puissancemini1) val1_largeur_grad = val_puissancemini2, temps1s_1 = 0, temps1mn_1 = 0, temps1h_1 = 0;
                    else if (val1_largeur_grad == val_puissancemini2) val1_largeur_grad = val_puissancemax, temps1s_1 = 0, temps1mn_1 = 0;

                    memo1_val_puissance = val1_largeur_grad;
                    indic0bits.front5 = high;
                    etape_entree = 3;
                    //break;

                    case 3:	// Minuterie
                        // Etape d'attente
                        if (val1_largeur_grad == val_puissancemini2)
                        {
                            if (temps1mn_1 == 60) temps1h_1++, temps1s_1 = 0, temps1mn_1 = 0;
                            if ((temps1h_1 == 3) && (temps1mn_1 == 30))
                            {
                                etape_entree = 4;
                                temps1ms_1 = 0;
                            }
                        }
                        else
                        {
                            if ((temps1mn_1 == val_duree_puissancemax) && (val1_largeur_grad == val_puissancemax) && !indic0bits.front1)
                            {
                                indic0bits.front1 = high;
                                val1_largeur_grad = val_mipuissance;
                            }
                            if (temps1mn_1 >= (val_duree_entree + val_duree_puissancemax))
                            {
                                if (val1_largeur_grad == val_puissancemax) val1_largeur_grad = val_mipuissance;
                                indic0bits.front5 = high;
                                etape_entree = 4;
                                temps1ms_1 = 0;
                            }
                        }
                        break;

                    case 4:	// Arrt dgressif
                        if ((temps1ms_1 == val_arret_degressif) && (val1_largeur_grad < val_puissancemini))
                        {
                            val1_largeur_grad++;
                            temps1ms_1 = 0;
                        }
                        else if (val1_largeur_grad >= val_puissancemini) etape_entree = 5, indic0bits.front5 = high;
                        break;

                    case 5: // Arrt
                        led_entree_off();
                        grad_entree_off();
                        etape_entree = 0;
                        indic0bits.front5 = high;
                        indic0bits.front1 = low;
                        break;

                    default:
                        etape_entree = 0;
                        break;
            }
            chemin = cycle;

            if (!in_memo)
            {
                Delay10KTCYx(160);	// Delay 160 x 10000 = 1600,000 cycles; 200ms @ 32MHz
                if(!in_memo)
                {
                    //initialisation adc
                    ADC_Init();
                    val_duree_couloir = v_ad(0);
                    val_duree_entree = v_ad(1);
                    adc_off();
                    Write_b_eep(2, val_duree_couloir);
                    Write_b_eep(3, val_duree_entree);
                    while (!in_memo) clrwdt();
                    chemin = debut;
                }
            }
            // Aiguillage
        } while (chemin == cycle);	// boucle 1
    } while (chemin == debut);	// boucle 2
}


/** FONCTIONS ******************************************************/
#include "interrupt.c"
#include "timebase.c"
#include "grad.c"
#include "fct.c"
